"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.DefaultWriter = void 0;
const fecha_1 = __importDefault(require("fecha"));
/**
 * 단순 console 로그
 */
class DefaultWriter {
    _message(type, msg) {
        console.log(`\n${type}: ` + msg);
    }
    log(msg) {
        this._message('log', msg);
    }
    error(msg) {
        this._message('err', msg);
    }
    write(msg) {
        console.log(msg);
    }
    get link() {
        return this._other;
    }
    set link(other) {
        this._other = other;
    }
}
exports.DefaultWriter = DefaultWriter;
const myMask = 'YYYY-MM-DD HH:mm:ss.SSS';
let _writer = new DefaultWriter();
let _format = ':time: > :name: - :msg:';
/**
 * Log
 */
class Logger {
    /**
     * Log
     * @param name Log 이름
     */
    constructor(name = '') {
        this._name = name;
    }
    static get writer() {
        return _writer;
    }
    static set writer(writer) {
        _writer = writer;
    }
    static get format() {
        return _format;
    }
    static set format(format) {
        _format = format;
    }
    /**
     * Log 이름. 아니면 클래스 이름
     */
    get name() {
        if (this._name !== '') {
            return this._name;
        }
        else {
            return this.constructor.name;
        }
    }
    _makeMsg(msgs) {
        const time = fecha_1.default.format(new Date(), myMask);
        const suitableMsgs = msgs.map(m => {
            if (typeof (m) === 'string') {
                return m;
            }
            else if (m instanceof Error) {
                return `${m.name}: ${m.message}\n${m.stack}`;
            }
            else {
                try {
                    return JSON.stringify(m, null, 2);
                }
                catch (err) {
                    return m;
                }
            }
        });
        return _format.replace(':time:', time)
            .replace(':name:', this.name)
            .replace(':msg:', suitableMsgs.join('\n'));
    }
    log(...msgs) {
        const msg = this._makeMsg(msgs);
        let writer = Logger.writer;
        while (writer) {
            writer.log(msg);
            writer = writer.link;
        }
    }
    error(...msgs) {
        const msg = this._makeMsg(msgs);
        let writer = Logger.writer;
        while (writer) {
            writer.error(msg);
            writer = writer.link;
        }
    }
}
exports.Logger = Logger;
//# sourceMappingURL=Logger.js.map