"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Timer = void 0;
/**
 * Timer
 */
class Timer {
    /**
     * Timer
     * @param interval interval 시간간격 안에 등록된 callback들이 차례대로 실행된다.
     * @param minTick callback과 callback사이에 최소 시간간격을 설정할 수 있다.
     */
    constructor(interval, minTick) {
        this.interval = interval;
        this.minTick = minTick;
        this._iterateList = [];
        this._onceList = [];
        this._id = 0;
        this._t = null;
        this._alter = true;
        if (!minTick) {
            this.minTick = 0;
        }
    }
    iterate(callback, ...args) {
        this._iterateList.push({
            id: ++this._id,
            callback,
            args,
        });
        this.start();
        return this._id;
    }
    remove(id) {
        let before = this._iterateList;
        this._iterateList = this._iterateList.filter(o => {
            return o.id !== id;
        });
        return before.length !== this._iterateList.length;
    }
    once(target, ...args) {
        return new Promise((resolve, reject) => {
            this._onceList.push({
                id: 0,
                callback: function (...args) {
                    target(...args).then(value => {
                        resolve(value);
                    }).catch(err => {
                        reject(err);
                    });
                },
                args,
            });
            this.start();
        });
    }
    start() {
        this.stop();
        let count = this._iterateList.length + this._onceList.length;
        if (count === 0) {
            return;
        }
        let tick = this.interval / count;
        if (this.minTick) {
            tick = Math.max(tick, this.minTick);
        }
        this._t = setInterval(this._do.bind(this), tick);
    }
    stop() {
        if (this._t) {
            clearInterval(this._t);
            this._t = null;
        }
    }
    pause(ms) {
        return new Promise((resolve, reject) => {
            this.stop();
            setTimeout(() => {
                this.start();
                resolve();
            }, ms);
        });
    }
    get isStarted() {
        return !!this._t;
    }
    _nextCallback() {
        let pack;
        if (this._alter) {
            pack = this._iterateList.shift();
            if (pack) {
                this._iterateList.push(pack);
                this._alter = !this._alter;
            }
            else {
                pack = this._onceList.shift();
            }
        }
        else {
            pack = this._onceList.shift();
            if (pack) {
                this._alter = !this._alter;
            }
            else {
                pack = this._iterateList.shift();
                if (pack) {
                    this._iterateList.push(pack);
                }
            }
        }
        return pack;
    }
    _do() {
        let pack = this._nextCallback();
        if (pack) {
            pack.callback(...pack.args);
        }
        else {
            this.stop();
        }
    }
}
exports.Timer = Timer;
//# sourceMappingURL=Timer.js.map