"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLogger = exports.logger = void 0;
const _1 = require(".");
function logHelper(...args) {
    let w;
    if (args.length === 0) {
        w = new _1.DefaultWriter();
    }
    else if (typeof (args[0]) === 'string') {
        w = new _1.FileWriter(args[0], args[1]);
    }
    else {
        w = args[0];
    }
    const f = (msg) => {
        let suited;
        switch (typeof (msg)) {
            case 'string': {
                suited = msg;
                break;
            }
            case 'number': {
                suited = msg.toString();
                break;
            }
            case 'boolean': {
                suited = (msg) ? 'true' : 'false';
                break;
            }
            default: {
                if (msg === undefined) {
                    suited = 'undefined';
                }
                else if (msg === null) {
                    suited = 'null';
                }
                else if (msg instanceof Error) {
                    suited = JSON.stringify({
                        name: msg.name,
                        message: msg.message,
                        stack: msg.stack,
                    }, null, 2);
                }
                else {
                    try {
                        suited = JSON.stringify(msg, null, 2);
                    }
                    catch (err) {
                        suited = '지원하는 type이 아니다.';
                    }
                }
            }
        }
        let ww = w;
        while (ww) {
            ww.write(suited);
            ww = ww.link;
        }
    };
    f.writer = w;
    return f;
}
// export function logger(...args: any[]) {
//   const helper = logHelper(...args)
//   return (target: any, property: string, descriptor: PropertyDescriptor) => {
//     const method = descriptor.value
//     descriptor.value = (...args) => {
//       const chunk = method.apply(this, args)
//       helper(chunk)
//       return chunk
//     }
//     descriptor.value.writer = helper['writer']
//   }
// }
function logger(...args) {
    let writer;
    if (args.length === 0) {
        writer = new _1.DefaultWriter();
    }
    else if (typeof (args[0]) === 'string') {
        writer = new _1.FileWriter(args[0], args[1]);
    }
    else {
        writer = args[0];
    }
    return (target, property, descriptor) => {
        const method = descriptor.value;
        descriptor.value = function (...args) {
            const chunk = method.apply(this, args);
            let w = writer;
            while (w) {
                w.write(chunk);
                w = w.link;
            }
            return chunk;
        };
        descriptor.value.writer = writer;
    };
}
exports.logger = logger;
function getLogger(...args) {
    return logHelper(...args);
}
exports.getLogger = getLogger;
//# sourceMappingURL=logger.v2.js.map