"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.promisify = void 0;
/**
 * Callback 방식을 Promise 로 변환
 * @param func callback 함수
 * @param hasErr error가 있다면 true, 아니면 false
 * @param ctx context 객체
 * @returns Promise
 */
function promisify(func, hasErr = true, ctx = undefined) {
    if (typeof hasErr !== 'boolean') {
        [hasErr, ctx] = [ctx, hasErr];
        if (hasErr === undefined) {
            hasErr = true;
        }
    }
    if (typeof hasErr !== 'boolean') {
        throw new Error('argument 전달이 잘못됐다.');
    }
    return function (...args) {
        return new Promise((resolve, reject) => {
            const ff = func.bind((ctx !== undefined) ? ctx : this);
            try {
                ff(...args, (err, ...args) => {
                    if (hasErr && err) {
                        reject(err);
                    }
                    else if (hasErr) {
                        resolve((args.length <= 1) ? args[0] : [...args]);
                    }
                    else {
                        resolve((args.length == 0) ? err : [err, ...args]);
                    }
                });
            }
            catch (err) {
                if (hasErr) {
                    reject(err);
                }
                else {
                    throw err;
                }
            }
        });
    };
}
exports.promisify = promisify;
exports.default = promisify;
//# sourceMappingURL=promisify.js.map